package com.example.happy_bird;

import javafx.application.Application;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;
import javafx.scene.control.Slider;
import javafx.scene.image.*;
import javafx.scene.image.Image;
import javafx.scene.input.KeyCode;
import javafx.scene.layout.*;
import javafx.scene.text.TextAlignment;
import javafx.stage.Stage;


import java.io.IOException;

public class HappyBirdMain extends Application {

    private Pane pane = new Pane();
    private Scene scene = new Scene(pane, 800, 700);

    private Label nameLabel = new Label("NAME");
    private Label acronymLabel = new Label("KÜRZEL");
    private Label headline = new Label("HAPPY BIRD");
    private Label soundLabel = new Label("SOUND");
    private Label currentScoreLabel = new Label("SCORE");
    private Label highscoreLabel = new Label("HIGHSCORE");
    private Label pauseMenu = new Label("Pause");
    private Label labels[] = {nameLabel, acronymLabel, soundLabel, currentScoreLabel, highscoreLabel};
    private Label headlineLabels[] = {headline};

    private Button startButton = new Button("START");
    private Button highscoresButton = new Button("HIGHSCORES");
    private Button settingsButton = new Button("EINSTELLUNGEN");
    private Button startGameButton = new Button("SPIEL STARTEN");
    private Button menuButton = new Button("ZURÜCK");
    private Button pauseButton = new Button("PAUSE");
    private Button continueButton = new Button("WEITERSPIELEN");

    private Button[] buttons = {startButton, highscoresButton, settingsButton, startGameButton, menuButton, pauseButton, continueButton};

    private TextField name = new TextField();
    private TextField acronym = new TextField();

    private TextField inputFields[] = {name, acronym};

    private Slider soundSlider = new Slider();

    private Image bird = new Image("file:src/main/resources/com/example/happy_bird/pics/bird.png", 50, 35, false, false);

    private Image upperPipe = new Image("file:src/main/resources/com/example/happy_bird/pics/PipeOben.png", 86, 300, false, false);
    private Image lowerPipe = new Image("file:src/main/resources/com/example/happy_bird/pics/PipeUnten.png", 86, 300, false, false);

    private Pane pause = new Pane();

    private boolean gameRuns;



    public static void main(String[] args) {
        launch(args);
    }

    @Override
    public void start(Stage stage) throws IOException {

        stage.setTitle("Happy Bird");
        stage.setScene(scene);

        generateMenueScene();

        gameRuns = false;

        /**ClickEvents:*/
        startButton.setOnAction(event ->  {
            startButtonClick();
        });

        settingsButton.setOnAction(event -> {
            settingsButtonClick();
        });

        highscoresButton.setOnAction(event -> {
            highscoresButtonClick();
        });

        menuButton.setOnAction(event -> {
            menuButtonClick();
        });

        startGameButton.setOnAction(event ->  {
            gameRuns = true;
            startGameButtonClick();
        });



        stage.show();
    }


    public void generateMenueScene() {
        /**Bild für Hintergrund erstellen:*/
        Image image = new Image("file:src/main/resources/com/example/happy_bird/pics/Background.png");

        /**Hintergrund erstellen:*/
        BackgroundImage backgroundImage = new BackgroundImage(
                image,
                BackgroundRepeat.NO_REPEAT,
                BackgroundRepeat.NO_REPEAT,
                BackgroundPosition.DEFAULT,
                BackgroundSize.DEFAULT
        );

        /**Hintergrund zu Pane hinzufügen:*/
        Background background = new Background(backgroundImage);
        pane.setBackground(background);



        /**Labels erstellen:*/
        //nameLabel = new Label("NAME");
        //acronymLabel = new Label("KÜRZEL");
        //headline = new Label("HAPPY BIRD");
        //soundLabel = new Label("SOUND");
        //currentScoreLabel = new Label("SCORE");
        //highscoreLabel = new Label("HIGHSCORE");


        /**Labels in Arrays speichern:*/


        /**Labels formatieren:*/
        labelsFormating(labels, pane);
        headlineFormating(headlineLabels, pane);

        /**alle labels an ihre Stelle:*/
        int yPositionLabels = 225;
        for (Label label : labels) {
            label.setLayoutX(275);
            label.setLayoutY(yPositionLabels);
            yPositionLabels += 150;
        }

        /**headline an ihre Stelle:*/
        headline.setLayoutX(250);
        headline.setLayoutY(100);

        /**soundLabel an seine Stelle:*/
        soundLabel.setLayoutY(225);



        /**Buttons erstellen:*/
        //startButton = new Button("START");
        //highscoresButton = new Button("HIGHSCORES");
        //settingsButton = new Button("EINSTELLUNGEN");
        //startGameButton = new Button("SPIEL STARTEN");
        //menuButton = new Button("ZURÜCK");

        /**Buttons in Array speichern:*/


        /**Buttons formatieren:*/
        buttonsFormating(buttons, pane);

        /**alle Buttons an ihre Stelle:*/
        int yPositionButtons = 225;
        for (Button button : buttons) {
            button.setLayoutX(300);
            button.setLayoutY(yPositionButtons);
            yPositionButtons += 100;
        }

        /**menuButton an seine Stelle und vorerst ausblenden:*/
        menuButton.setLayoutX(575);
        menuButton.setLayoutY(625);
        menuButton.setVisible(false);

        /**startGameButton unsichtbar*/
        startGameButton.setVisible(false);

        /**continueButton unsichtbar*/
        continueButton.setVisible(false);



        /**Textfelder erstellen:*/
        //name = new TextField();
        //acronym = new TextField();

        /**Textfelder in Array speichern:*/


        /**Textfelder formatieren:*/
        textfieldsFormating(inputFields, pane);

        /**Textfelder an ihre Position:*/
        int yPositionInputFields = 290;
        for (TextField feld : inputFields) {
            feld.setLayoutX(275);
            feld.setLayoutY(yPositionInputFields);
            yPositionInputFields += 150;
        }



        /**SoundSlider erstellen:*/
        //soundSlider = new Slider();

        /**SoundSlicer anpassen:*/
        soundSliderFormating(soundSlider, pane);

    }

    /**
     * Generiert den GameScreen durch Ausblenden und
     * Sichtbarmachen der richtigen Buttons, Labels, ...
     */
    public void generateGameScreen() {
        /**Headline-Label umbenennen*/
        nameLabel.setText("NAME: " + acronym.getText());

        /**Textfelder ausblenden*/
        for(TextField feld : inputFields) {
            feld.setVisible(false);
        }

        /**Labels ausblenden*/
        for (Label label : labels )
        {
            label.setVisible(false);
        }
        headline.setVisible(false);

        /**nameLabel, currentScoreLabel und highscoreLabel positionieren und sichtbar machen*/
        nameLabel.setLayoutX(10);
        nameLabel.setLayoutY(10);
        nameLabel.setVisible(true);

        currentScoreLabel.setLayoutX(540);
        currentScoreLabel.setLayoutY(10);
        currentScoreLabel.setVisible(true);

        highscoreLabel.setLayoutX(540);
        highscoreLabel.setLayoutY(70);
        highscoreLabel.setVisible(true);

        /**Übrige Buttons ausblenden*/
        startGameButton.setVisible(false);
        menuButton.setVisible(false);

        /**Pause-Button positionieren und sichtbar machen*/
        pauseButton.setLayoutX(575);
        pauseButton.setLayoutY(625);
        pauseButton.setVisible(true);

        /**Vogel hinzufügen, bisher nur als Bild*/
        ImageView birdView = new ImageView(bird);
        pane.getChildren().add(birdView);
        birdView.setLayoutY(325);
        birdView.setLayoutX(50);

        /**Pipes oben, unten hinzufügen, bisher nur als Bild*/
        ImageView upperPipeView = new ImageView(upperPipe);
        pane.getChildren().add(upperPipeView);
        upperPipeView.setLayoutX(400);
        upperPipeView.setLayoutY(0);

        ImageView lowerPipeView = new ImageView(lowerPipe);
        pane.getChildren().add(lowerPipeView);
        lowerPipeView.setLayoutX(400);
        lowerPipeView.setLayoutY(430);
    }

    /***
     * Formatiert alle Buttons und fügt sie der Pane hinzu
     * @param buttons   ButtonArray mit allen Arrays
     * @param pane      Pane, auf der die Buttons angezeigt werden
     */
    public void buttonsFormating(Button[] buttons, Pane pane) {
        for (Button button : buttons) {
            button.setPrefSize(200, 50);

            button.setStyle("-fx-background-color: #e86000; " +
                    "-fx-text-fill: #FFFFFF; " +
                    "-fx-font-size: 20px; " +
                    "-fx-border-width: 5px;" +
                    "-fx-border-color: #FFFFFF;" +
                    "-fx-font-weight: bold;" +
                    "-fx-border-radius: 15px;" +
                    "-fx-background-radius: 20px;");

            pane.getChildren().add(button);
        }
    }

    /***
     * Formatiert alle kleinen Labels (keine Überschrift) und fügt sie der Pane hinzu
     * @param labels    LabelArray mit allen Labels
     * @param pane      Pane, auf der die Labels angezeigt werden
     */
    public void labelsFormating(Label[] labels, Pane pane) {
        for (Label label : labels) {
            label.setPrefSize(250, 50);

            label.setStyle("-fx-background-color: #DED894; " +
                    "-fx-text-fill: #000000; " +
                    "-fx-font-size: 20px; " +
                    "-fx-border-width: 5px;" +
                    "-fx-border-color: #543847;" +
                    "-fx-font-weight: bold;" +
                    "-fx-alignment: center;" +
                    "-fx-background-radius: 20px;" +
                    "-fx-border-radius: 15px;");

            pane.getChildren().add(label);

            label.setVisible(false);
        }
    }

    /***
     * Formatiert alle Überschriften-Labels und fügt sie der Pane hinzu
     * @param labels    LabelArray mit Überschriften-Labels
     * @param pane      Pane, auf der die Labels angezeigt werden
     */
    public void headlineFormating(Label[] labels, Pane pane) {
        for (Label label : labels) {
            label.setPrefSize(300, 75);
            label.setTextAlignment(TextAlignment.CENTER);

            label.setStyle("-fx-background-color: #DED894; " +
                    "-fx-text-fill: #000000; " +
                    "-fx-font-size: 35px; " +
                    "-fx-border-width: 5px;" +
                    "-fx-border-color: #543847;" +
                    "-fx-font-weight: bold;" +
                    "-fx-alignment: center;" +
                    "-fx-background-radius: 20px;" +
                    "-fx-border-radius: 15px;");

            pane.getChildren().add(label);
        }
    }

    /***
     * Formatiert alle Textfelder und fügt sie der Pane hinzu
     * @param textfields    TextFieldArray mit allen Textfeldern
     * @param pane          Pane, auf der die Textfelder angezeigt werden
     */
    public void textfieldsFormating(TextField[] textfields, Pane pane) {
        for (TextField field : textfields) {
            field.setPrefSize(250, 50);
            field.setStyle("-fx-font-size: 20px;" +
                    "-fx-font-weight: bold;");

            pane.getChildren().add(field);

            field.setVisible(false);
        }
    }

    /***
     * Formatiert den soundSlider und fügt ihn der Pane hinzu
     * @param slider    SoundSlider
     * @param pane      Pane, auf der der Slider angezeigt wird
     */
    public void soundSliderFormating(Slider slider, Pane pane) {
        slider.setPrefSize(200, 5);
        slider.setLayoutX(300);
        slider.setLayoutY(295);

        pane.getChildren().add(slider);

        slider.setVisible(false);
    }

    /***
     * Formatiert den PauseScreen
     */
    public void pauseScreenFormating() {
        pause.setLayoutX(200);
        pause.setLayoutY(100);
        pause.setPrefSize(400, 500);
        pause.setStyle("-fx-background-color: #DED894; " +
                "-fx-border-width: 5px;" +
                "-fx-border-color: #543847;" +
                "-fx-border-radius: 15px;" +
                "-fx-background-radius: 20px;");
    }

    /***
     * Hauptmenü wird ausgeblendet, Nameneingabe folgt
     */
    public void startButtonClick() {
        headline.setText("START");
        startGameButton.setOnAction(event ->  {
            startGameButtonClick();
        });
        for(Button button : buttons) {
            button.setVisible(false);
        }

        startGameButton.setVisible(true);
        menuButton.setVisible(true);
        startGameButton.setLayoutY(520);
        for(TextField feld : inputFields) {
            feld.setVisible(true);
        }
        for (Label label : labels ) {
            label.setVisible(true);
        }
        highscoreLabel.setVisible(false);
        currentScoreLabel.setVisible(false);
        soundLabel.setVisible(false);

    }


    /***
     * Hauptmenü wird ausgeblendet, Einstellungen folgen
     */
    public void settingsButtonClick() {
        headline.setText("EINSTELLUNGEN");
        startButton.setVisible(false);
        highscoresButton.setVisible(false);
        settingsButton.setVisible(false);
        menuButton.setVisible(true);
        soundSlider.setVisible(true);
        soundLabel.setVisible(true);
    }

    /***
     * Hauptmenü wird ausgeblendet, Highscores folgen
     */
    public void highscoresButtonClick() {
        headline.setText("HIGHSCORES");
        startButton.setVisible(false);
        highscoresButton.setVisible(false);
        settingsButton.setVisible(false);
        menuButton.setVisible(true);
    }

    /***
     * Nameneingabe, Einstellungen und Highscores werden ausgeblendet
     * Erneutes Anzeigen des Hauptmenüs
     */
    public void menuButtonClick() {
        headline.setText("HAPPY BIRD");
        startButton.setVisible(true);
        highscoresButton.setVisible(true);
        settingsButton.setVisible(true);
        menuButton.setVisible(false);
        soundSlider.setVisible(false);
        soundLabel.setVisible(false);
        startGameButton.setVisible(false);

        for(TextField feld : inputFields) {
            feld.setVisible(false);
        }

        for (Label label : labels ) {
            label.setVisible(false);
        }
    }

    /***
     * Startet Spiel nach Namenseingabe
     */
    public void startGameButtonClick() {
        generateGameScreen();


        /**Klick auf den Pause-Button, um das Pause-Menü aufzurufen*/
        pauseButton.setOnAction(event -> {
            //
            if(gameRuns) {
                gameRuns = false;
            } else {
                gameRuns = true;
            }
            pauseButtonClick(gameRuns);
        });

        /**ESC-Tast drücken, um das Pause-Menü aufzurufen*/
        scene.setOnKeyPressed(e -> {
            if(e.getCode() == KeyCode.ESCAPE) {
                if(gameRuns) {
                    gameRuns = false;
                    pauseButtonClick(false);
                } else {
                    gameRuns = true;
                    pauseButtonClick(true);
                }
            }
        });
    }

    /***
     * Ruft das Pause-Menü im Spiel auf oder blendet es wieder aus
     */
    public void pauseButtonClick(Boolean gameRuns) {
        pauseScreenFormating();

        /**PauseScreen aufrufen, wenn das Spiel läuft, sonst wieder ausblenden*/
        if(gameRuns) {
            /**PauseScreen ausblenden*/
            pause.setVisible(true);
            pauseButton.setVisible(false);
            highscoresButton.setVisible(true);
            settingsButton.setVisible(true);
            highscoresButton.setLayoutX(100);
            settingsButton.setLayoutX(100);

            /**Zum PauseScreen hinzufügen*/
            pause.getChildren().add(settingsButton);
            pause.getChildren().add(highscoresButton);
            pane.getChildren().add(pause);
        } else {
            /**PauseScreen aufrufen/ sichtbar machen*/
            pause.setVisible(false);

            pauseButton.setVisible(true);
            highscoresButton.setVisible(false);
            settingsButton.setVisible(false);
            highscoresButton.setLayoutX(300);
            settingsButton.setLayoutX(300);

            /**Vom PauseScreen entfernen*/
            pause.getChildren().remove(settingsButton);
            pause.getChildren().remove(highscoresButton);
            pane.getChildren().remove(pause);
        }
    }
}
